@@ -82,6 +82,10 @@ public class MailCto {
 
     private final String id;
 
+    private final String rootMailId;
+
+    private final String folderId;
+
     @JsonInclude(JsonInclude.Include.NON_EMPTY)
     private final MailFlagsCto flags;
 
@@ -121,13 +125,18 @@ public class MailCto {
 
     private final Long size;
 
+    // Deprecated. Please use 'hasAttachment'.
     private final Boolean realAttachment;
 
+    private final Boolean hasAttachment;
+
     @JsonInclude(JsonInclude.Include.NON_EMPTY)
     private final List<AttachmentCto> attachments;
 
     public MailCto(Mail mail) {
         id = mail.getId();
+        rootMailId = mail.getRootMailId();
+        folderId = mail.getFolderId();
         flags = new MailFlagsCto(mail.getFlags());
         color = mail.getColor();
         receivedDate = mail.getReceivedDate();
@@ -146,7 +155,8 @@ public class MailCto {
         List<AttachmentCto> attachments = ListUtil.map(mail.getParts(), partMapFunction);
         body = getBodyOrNull(attachments);
         bodyType = getBodyTypeOrNull(attachments);
-        realAttachment = mail.getHasAttachments();
+        hasAttachment = mail.getHasAttachments() == Boolean.TRUE ? Boolean.TRUE : null;
+        realAttachment = hasAttachment;
         this.attachments = getAttachmentListWithoutBodyOrNull(attachments);
     }
 
@@ -169,7 +179,9 @@ public class MailCto {
             return null;
         }
         for (AttachmentCto part : parts) {
-            if (!"attachment".equals(part.getDisposition())) {
+            //if (!"attachment".equals(part.getDisposition())) {
+            if (!("attachment".equals(part.getDisposition()) ||
+                    ("inline".equals(part.getDisposition()) && !("text/plain".equals(part.getContentType()) ||"text/html".equals(part.getContentType()))))) {
                 return part;
             }
         }
@@ -182,7 +194,9 @@ public class MailCto {
         }
         for (Iterator<AttachmentCto> it = attachments.iterator(); it.hasNext(); ) {
             AttachmentCto attachment = it.next();
-            if (!"attachment".equals(attachment.getDisposition())) {
+            //if (!"attachment".equals(attachment.getDisposition())) {
+            if (!("attachment".equals(attachment.getDisposition()) ||
+                    ("inline".equals(attachment.getDisposition()) && !("text/plain".equals(attachment.getContentType()) ||"text/html".equals(attachment.getContentType()))))) {
                 it.remove();
             }
         }
