@@ -55,44 +55,44 @@ import java.util.List;
 import com.openexchange.mobile.api.facade.auth.SessionData;
 import com.openexchange.mobile.api.facade.connectors.ConnectorFactory;
 import com.openexchange.mobile.api.facade.connectors.ConnectorFactoryFactory;
-import com.openexchange.mobile.api.facade.connectors.JsLob;
-import com.openexchange.mobile.api.facade.connectors.impl.AccountsConnector;
 import com.openexchange.mobile.api.facade.connectors.impl.AllFoldersConnector;
 import com.openexchange.mobile.api.facade.connectors.impl.ClearFolderConnector;
 import com.openexchange.mobile.api.facade.connectors.impl.ConfigConnector;
 import com.openexchange.mobile.api.facade.connectors.impl.CreateFolderConnector;
 import com.openexchange.mobile.api.facade.connectors.impl.DeleteFolderConnector;
 import com.openexchange.mobile.api.facade.connectors.impl.ExamineMultipleFoldersConnector;
-import com.openexchange.mobile.api.facade.connectors.impl.GetJsLobConnector;
-import com.openexchange.mobile.api.facade.connectors.impl.UserConnector;
-import com.openexchange.mobile.api.facade.connectors.responses.ArrayDataResponseMto;
+import com.openexchange.mobile.api.facade.connectors.impl.UpdateFolderConnector;
 import com.openexchange.mobile.api.facade.connectors.responses.ConfigResponseMto;
 import com.openexchange.mobile.api.facade.connectors.responses.ExamineResponseMto;
 import com.openexchange.mobile.api.facade.connectors.responses.FoldersResponseMto;
-import com.openexchange.mobile.api.facade.connectors.responses.RawResponseMto;
 import com.openexchange.mobile.api.facade.connectors.responses.StringResponseMto;
-import com.openexchange.mobile.api.facade.connectors.responses.UserResponseMto;
-import com.openexchange.mobile.api.facade.connectors.responses.mtos.AccountMto;
 import com.openexchange.mobile.api.facade.connectors.responses.mtos.ExamineResultMto;
 import com.openexchange.mobile.api.facade.connectors.responses.mtos.FoldersMto;
-import com.openexchange.mobile.api.facade.connectors.responses.mtos.UserMto;
 import com.openexchange.mobile.api.facade.exceptions.ApiFacadeException;
 import com.openexchange.mobile.api.facade.models.Account;
 import com.openexchange.mobile.api.facade.models.Config;
 import com.openexchange.mobile.api.facade.models.CreatedFolder;
 import com.openexchange.mobile.api.facade.models.ExamineResult;
 import com.openexchange.mobile.api.facade.models.Folder;
-import com.openexchange.mobile.api.facade.utils.JsonUtil;
+import com.openexchange.mobile.api.facade.models.MailAccountData;
+import com.openexchange.mobile.api.facade.models.UpdatedFolder;
+import com.openexchange.mobile.api.facade.models.Folder.Type;
 import com.openexchange.mobile.api.facade.utils.ListUtil;
 import com.openexchange.mobile.api.facade.utils.MapFunction;
 
 import lombok.RequiredArgsConstructor;
+import lombok.Setter;
+import lombok.extern.slf4j.Slf4j;
 
 @RequiredArgsConstructor
+@Slf4j
 public class FoldersService {
 
     private final ConnectorFactoryFactory connectorFactoryFactory;
 
+    @Setter
+    private AccountsService accountsService;
+
     public CreatedFolder createFolder(SessionData sessionData, String parentFolderId, String name) {
         ConnectorFactory connectorFactory = connectorFactoryFactory.getConnectorFactory();
         CreateFolderConnector connector = connectorFactory.getCreateFolderConnector(sessionData, parentFolderId, name);
@@ -101,6 +101,14 @@ public class FoldersService {
         return new CreatedFolder(folderId, name);
     }
 
+    public UpdatedFolder updateFolder(SessionData sessionData, String folderId, String newParentFolderId, String newName) {
+        ConnectorFactory connectorFactory = connectorFactoryFactory.getConnectorFactory();
+        UpdateFolderConnector connector = connectorFactory.getUpdateFolderConnector(sessionData, folderId, newParentFolderId, newName);
+        StringResponseMto response = connector.execute();
+        String newFolderId = UpdateFolderConnector.map(response);
+        return new UpdatedFolder(newFolderId);
+    }
+
     public void deleteFolder(SessionData sessionData, String folderId) {
         ConnectorFactory connectorFactory = connectorFactoryFactory.getConnectorFactory();
         DeleteFolderConnector connector = connectorFactory.getDeleteFolderConnector(sessionData, folderId);
@@ -117,36 +125,49 @@ public class FoldersService {
         final ConnectorFactory connectorFactory = connectorFactoryFactory.getConnectorFactory();
         Config config = retrieveConfiguration(connectorFactory, sessionData);
         final boolean isNamespaceSet = "INBOX/".equals(config.getNamespace());
-        List<AccountMto> accountMtos = retrieveAccounts(connectorFactory, sessionData);
-        final UserMto userMto = retrieveUserMto(connectorFactory, sessionData);
-        return ListUtil.map(accountMtos, new MapFunction<AccountMto, Account>() {
+        List<Account> accounts = accountsService.getAccounts(connectorFactory, sessionData);
+        return ListUtil.map(accounts, new MapFunction<Account, Account>() {
 
             @Override
-            public Account map(AccountMto accountMto) {
-                return mapAccount(connectorFactory, sessionData, accountMto, userMto, language, isNamespaceSet);
+            public Account map(Account account) {
+                return addFoldersImpl(connectorFactory, sessionData, account, language, isNamespaceSet);
             }
 
         });
     }
 
-    private Account mapAccount(ConnectorFactory connectorFactory, SessionData sessionData, AccountMto accountMto, UserMto userMto, String language, boolean isNamespaceSet) {
-        List<Folder> folders = retrieveFolders(connectorFactory, sessionData, accountMto, language);
-        examineFolders(connectorFactory, sessionData, folders);
-        String defaultSendAddress;
-        if (accountMto.isPrimaryAccount()) {
-            defaultSendAddress = retrieveDefaultSendAddress(connectorFactory, sessionData);
-        } else {
-            defaultSendAddress = null;
+    public Account addFolders(ConnectorFactory connectorFactory, SessionData sessionData, Account account, String language) {
+        Config config = retrieveConfiguration(connectorFactory, sessionData);
+        boolean isNamespaceSet = "INBOX/".equals(config.getNamespace());
+        return addFoldersImpl(connectorFactory, sessionData, account, language, isNamespaceSet);
+    }
+
+    private Account addFoldersImpl(ConnectorFactory connectorFactory, SessionData sessionData, Account account, String language, boolean isNamespaceSet) {
+        List<Folder> folders = retrieveFolders(connectorFactory, sessionData, account.getRootFolderId(), account.getArchiveFolderId(), language);
+        if (!account.isPop3Account()) {
+            examineFolders(connectorFactory, sessionData, folders);
         }
-        Account account = new Account(accountMto, userMto, defaultSendAddress, folders);
-        adjustFolderParents(account, isNamespaceSet);
+        account = account.withFolders(folders);
+        adjustFolderParents(account, account.isPrimaryAccount() && isNamespaceSet);
         return account;
     }
 
-    private String retrieveDefaultSendAddress(ConnectorFactory connectorFactory, SessionData sessionData) {
-        GetJsLobConnector jsLobConnector = connectorFactory.getGetJsLobConnector(sessionData, JsLob.MAIL);
-        RawResponseMto response = jsLobConnector.execute();
-        return JsonUtil.getString(GetJsLobConnector.map(response), "defaultSendAddress");
+    public MailAccountData addFolders(ConnectorFactory connectorFactory, SessionData sessionData, MailAccountData account, String language) {
+        Config config = retrieveConfiguration(connectorFactory, sessionData);
+        boolean isNamespaceSet = "INBOX/".equals(config.getNamespace());
+        return addFoldersImpl(connectorFactory, sessionData, account, language, isNamespaceSet);
+    }
+
+    private MailAccountData addFoldersImpl(ConnectorFactory connectorFactory, SessionData sessionData, MailAccountData account, String language, boolean isNamespaceSet) {
+        List<Folder> folders = retrieveFolders(connectorFactory, sessionData, account.getRootFolderId(), account.getArchiveFolderId(), language);
+        if (!account.isPop3Account()) {
+            examineFolders(connectorFactory, sessionData, folders);
+        } else {
+            examineResetFolders(folders);
+        }
+        account = account.withFolders(folders);
+        adjustFolderParents(account, account.isPrimaryAccount() && isNamespaceSet);
+        return account;
     }
 
     private Config retrieveConfiguration(ConnectorFactory connectorFactory, SessionData sessionData) {
@@ -155,33 +176,21 @@ public class FoldersService {
         return ConfigConnector.map(response);
     }
 
-    private List<AccountMto> retrieveAccounts(ConnectorFactory connectorFactory, SessionData sessionData) {
-        AccountsConnector accountsConnector = connectorFactory.getAccountsConnector(sessionData);
-        ArrayDataResponseMto response = accountsConnector.execute();
-        Object[][] data = response.getData();
-        if (data == null || data.length == 0) {
-            throw ApiFacadeException.internalServerError("No account data available");
-        }
-        List<AccountMto> accounts = AccountsConnector.map(data);
-        if (sessionData.getConfiguration().isReturnNonPrimaryAccounts()) {
-            return accounts;
-        }
-        return Collections.singletonList(accounts.get(0));
-    }
-
-    private UserMto retrieveUserMto(ConnectorFactory connectorFactory, SessionData sessionData) {
-        UserConnector connector = connectorFactory.getUserConnector(sessionData);
-        UserResponseMto response = connector.execute();
-        return response.getData();
-    }
-
-    private List<Folder> retrieveFolders(ConnectorFactory connectorFactory, SessionData sessionData, AccountMto account, String language) {
+    private List<Folder> retrieveFolders(ConnectorFactory connectorFactory, SessionData sessionData, String rootFolderId, String archiveFolderId, String language) {
         boolean includeNonPrivateFolders = sessionData.getConfiguration().isIncludeNonPrivateFolders();
-        AllFoldersConnector connector = connectorFactory.getAllFoldersConnector(sessionData, account.getRootFolderId(), language);
-        FoldersResponseMto response = connector.execute();
-        FoldersMto data = response.getData();
+        AllFoldersConnector connector = connectorFactory.getAllFoldersConnector(sessionData, rootFolderId, language);
+        FoldersMto data;
+        try {
+            FoldersResponseMto response = connector.execute();
+            data = response.getData();
+        } catch (ApiFacadeException e) {
+            log.info("Retrieve folders failed with code: " + e.getErrorCode());
+            // Return fake INBOX folder to not crash older iOS clients.
+            Folder inboxFolder = new Folder(rootFolderId + "/INBOX", rootFolderId, "INBOX", Type.INBOX);
+            return Collections.singletonList(inboxFolder);
+        }
         return data != null
-                ? AllFoldersConnector.map(data.getPrivateFolders(), account.getArchiveFolderId(), includeNonPrivateFolders)
+                ? AllFoldersConnector.map(data.getPrivateFolders(), archiveFolderId, includeNonPrivateFolders)
                 : Collections.<Folder>emptyList();
     }
 
@@ -216,9 +225,38 @@ public class FoldersService {
         return new ExamineResult(folderId, changed, newValidity, newNext);
     }
 
+    private void examineResetFolders(List<Folder> folders) {
+        for (int index = 0; index < folders.size(); index++) {
+            Folder folder = folders.get(index);
+            folder.setValidity(null);
+            folder.setNext(null);
+            folder.setTotal(-1);
+            folder.setUnread(-1);
+        }
+    }
+
     private void adjustFolderParents(Account account, boolean isNamespaceSet) {
+        if (!isNamespaceSet) {
+            return;
+        }
         Folder inboxFolder = account.getStandardFolders().getInboxFolder();
-        if (inboxFolder != null && isNamespaceSet) {
+        if (inboxFolder != null) {
+            String inboxFolderId = inboxFolder.getId();
+            String parentFolderId = inboxFolder.getParentId();
+            for (Folder folder : account.getFolders()) {
+                if (inboxFolderId.equals(folder.getParentId())) {
+                    folder.setParentId(parentFolderId);
+                }
+            }
+        }
+    }
+
+    private void adjustFolderParents(MailAccountData account, boolean isNamespaceSet) {
+        if (!isNamespaceSet) {
+            return;
+        }
+        Folder inboxFolder = account.getStandardFolders().getInboxFolder();
+        if (inboxFolder != null) {
             String inboxFolderId = inboxFolder.getId();
             String parentFolderId = inboxFolder.getParentId();
             for (Folder folder : account.getFolders()) {
